package gs.config.v0.audit

import cats.effect.IO
import gs.config.v0.ConfigError
import gs.config.v0.ConfigKey
import gs.config.v0.ConfigName
import gs.config.v0.Configuration
import gs.config.v0.GsSuite
import gs.config.v0.source.ConfigSource

class AuditedConfigurationTests extends GsSuite:
  import AuditedConfigurationTests.*

  iotest(
    "should not return values, but should record attempts to find, when no config exists"
  ) {
    for
      config <- Configuration
        .audited(ConfigSource.inMemory[IO](Map.empty))
        .build()
      string   <- config.getValue(Keys.KString)
      manifest <- config.manifest.snapshot()
    yield
      assert(string == Left(ConfigError.MissingValue(Names.KString)))
      assert(
        manifest.get(Names.KString) == Some(
          List(
            ConfigQueryResult.Failure(
              sources = List(config.sources.head.name),
              error = ConfigError.MissingValue(Names.KString)
            )
          )
        )
      )
  }

object AuditedConfigurationTests:

  object Names:

    val KString: ConfigName = ConfigName("string")

  end Names

  object Keys:

    val KString: ConfigKey[String] = ConfigKey.Required[String](Names.KString)

  end Keys

end AuditedConfigurationTests
